@@ -20,6 +20,8 @@
 #define CHXJ_TRUE    (1)
 #define CHXJ_FALSE   (0)
 
+/* #define DEBUG_FD(X,args...) {FILE*fp=fopen("/tmp/error.log", "ab");fprintf(fp, X, ##args);fflush(fp);fclose(fp);} */
+
 #ifdef UNUSED
 #elif defined(__GNUC__)
 # define UNUSED(x) UNUSED_ ## x __attribute__((unused))
@@ -284,37 +286,62 @@ typedef struct mod_chxj_config mod_chxj_config;
 
 #if defined(USE_MYSQL_COOKIE)
 #  include "chxj_mysql.h"
-#elif defined(USE_MEMCACHE_COOKIE)
+#endif
+#if defined(USE_MEMCACHE_COOKIE)
 #  include "chxj_memcache.h"
 #endif
+
+/* cookie store type */
+#define CHXJ_COOKIE_STORE_TYPE_DBM      "dbm"
+#define CHXJ_COOKIE_STORE_TYPE_MYSQL    "mysql"
+#define CHXJ_COOKIE_STORE_TYPE_MEMCACHE "memcache"
+typedef enum {
+  COOKIE_STORE_TYPE_NONE     = 0,
+  COOKIE_STORE_TYPE_DBM      = 1,
+  COOKIE_STORE_TYPE_MYSQL    = 2,
+  COOKIE_STORE_TYPE_MEMCACHE = 3, 
+} cookie_store_type_t;
+
 struct mod_chxj_config {
   int                   image;
 
-  char*                 device_data_file;
-  char*                 emoji_data_file;
+  char                  *device_data_file;
+  char                  *emoji_data_file;
 
-  char*                 image_cache_dir;
-  char*                 image_copyright;
+  char                  *image_cache_dir;
+  char                  *image_copyright;
   unsigned long         image_cache_limit;
-  device_table_list*    devices;
-  emoji_t*              emoji;
-  emoji_t*              emoji_tail;
-  char*                 server_side_encoding;
+  device_table_list     *devices;
+  emoji_t               *emoji;
+  emoji_t               *emoji_tail;
+  char                  *server_side_encoding;
 
-  char*                 dir; /* for LOG */
+  char                  *dir; /* for LOG */
 
-  apr_array_header_t*   convrules;
+  apr_array_header_t    *convrules;
 
-  char*                 cookie_db_dir;
+  char                  *cookie_db_dir;
   long                  cookie_timeout;
+  cookie_store_type_t   cookie_store_type;
+  int                   cookie_lazy_mode;
 
 #if defined(USE_MYSQL_COOKIE)
   mysql_t               mysql;
-#elif defined(USE_MEMCACHE_COOKIE)
+#endif
+#if defined(USE_MEMCACHE_COOKIE)
   memcache_t            memcache;
 #endif
 };
 
+#define IS_COOKIE_STORE_DBM(X)      ((X) == COOKIE_STORE_TYPE_DBM)
+#define IS_COOKIE_STORE_MYSQL(X)    ((X) == COOKIE_STORE_TYPE_MYSQL)
+#define IS_COOKIE_STORE_MEMCACHE(X) ((X) == COOKIE_STORE_TYPE_MEMCACHE)
+#define IS_COOKIE_STORE_NONE(X)     ((X) == COOKIE_STORE_TYPE_NONE)
+
+#define COOKIE_LAZY_ON                (2)
+#define COOKIE_LAZY_OFF               (1)
+#define IS_COOKIE_LAZY(X)             ((X)->cookie_lazy_mode == COOKIE_LAZY_ON)
+
 
 #define CONVRULE_ENGINE_ON_BIT        (0x00000001)
 #define CONVRULE_ENGINE_OFF_BIT       (0x00000002)
@@ -333,14 +360,18 @@ struct mod_chxj_config {
 #define CONVRULE_PC_FLAG_ON_BIT       (0x00000001)
 #define CONVRULE_PC_FLAG_OFF_BIT      (0x00000002)
 
+
 typedef struct {
   apr_global_mutex_t* cookie_db_lock;
 } mod_chxj_global_config;
 
 typedef struct {
   apr_size_t len;
+  chxjconvrule_entry  *entryp;
+  device_table        *spec;
 
   apr_bucket_brigade *bb;
+  apr_pool_t *pool;
 
   char* buffer;
 } mod_chxj_ctx;
@@ -350,6 +381,7 @@ typedef struct {
 #define CHXJ_MOD_CONFIG_KEY   "chxj_module_key"
 
 #define HTTP_USER_AGENT       "User-Agent"
+#define HTTP_HOST             "Host"
 #define CHXJ_HTTP_USER_AGENT  "CHXJ_HTTP_USER_AGENT"
 
 module AP_MODULE_DECLARE_DATA chxj_module;
