@@ -190,7 +190,7 @@ chxj_headers_fixup(request_rec *r)
  * @param len [i/o] It is length of former HTML character string. 
  */
 static char* 
-chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *spec, const char *ua)
+chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *spec, const char *ua, cookie_t **cookiep)
 {
   char                *user_agent;
   char                *dst;
@@ -287,6 +287,9 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *s
     *len = 1;
   }
   dst[*len] = 0;
+  if (cookie) {
+    *cookiep = cookie;
+  }
 
   DBG(r, "end of chxj_exchange()");
 
@@ -622,7 +625,7 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
   char*               user_agent = NULL;
   apr_size_t          len;
   mod_chxj_ctx*       ctx = (mod_chxj_ctx *)f->ctx;
-  cookie_t*           cookie;
+  cookie_t*           cookie = NULL;
   char*               location_header;
   mod_chxj_config*    dconf;
   chxjconvrule_entry  *entryp = NULL;
@@ -756,7 +759,7 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
                                         (const char**)&tmp, 
                                         (apr_size_t*)&ctx->len,
                                         spec,
-                                        user_agent);
+                                        user_agent, &cookie);
 
 #if 0
             DBG(r, "output data=[%.*s]", ctx->len,ctx->buffer);
@@ -769,7 +772,7 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
                                         (const char**)&ctx->buffer, 
                                         (apr_size_t*)&ctx->len,
                                         spec,
-                                        user_agent);
+                                        user_agent, &cookie);
 
           }
         }
@@ -865,6 +868,16 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
         apr_table_setn(r->headers_out, "Content-Length", contentLength);
         
         if (ctx->len > 0) {
+          DBG(r, "call pass_data_to_filter()");
+          location_header = (char*)apr_table_get(r->headers_out, "Location");
+          if (cookie && location_header) {
+            DBG(r, "Location Header=[%s]", location_header);
+            location_header = chxj_add_cookie_parameter(r,
+                                                        location_header,
+                                                        cookie);
+            apr_table_setn(r->headers_out, "Location", location_header);
+            DBG(r, "Location Header=[%s]", location_header);
+          }
           rv = pass_data_to_filter(f, 
                                    (const char*)ctx->buffer, 
                                    (apr_size_t)ctx->len);
@@ -898,7 +911,7 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
              * Location Header Check to add cookie parameter.
              */
             location_header = (char*)apr_table_get(r->headers_out, "Location");
-            if (location_header) {
+            if (cookie && location_header) {
               DBG(r, "Location Header=[%s]", location_header);
               location_header = chxj_add_cookie_parameter(r,
                                                           location_header,
@@ -914,6 +927,7 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
         }
 
         apr_table_setn(r->headers_out, "Content-Length", "0");
+        DBG(r, "call pass_data_to_filter()");
         rv = pass_data_to_filter(f, (const char*)"", (apr_size_t)0);
 
         return rv;
